;+
; NAME:
;	sun_today
; PURPOSE:
;       create sun_today image for showpix (guts from first_light)
;       read the existing genx file version by default (speed)
;
; Calling Sequence:
;    sun_today, suntoday, r, g, b,/generate	; create new image (slow)
;    sun_today, suntoday, r, g, b	      ; read existing showpix copy (faster)
;
; Input Parameters: 
;   none
;
; Output Parameters:
;   suntoday - collage of SXT and groundbase images [bytarr(850,850)]
; 
; CATEGORY:
; MODIFICATION HISTORY:
;       ------- edited modification history for first_light -------
;	HSH, written July 1993
;       da       30-Apr-94, Produces collage of SXT + 4 GBO images
;       BNH       6-Jul-94, Changed display stuff for collage to 
;			    (hopefully) better deal with varying
;                           background in gki, gkm images
;       SLF  - 10-Jul-1994 Broke out collage, wrote to Z-buffer 
;			   return the collage and color table vectors
;       SLF  - 12-Jul-1994 Scale character size (Z ne X), allow read of existing
;       SLF  -  6-Sep-1994 Eliminate offpoint check
;       SLF  - 15-Sep-1994 Only reposition GBO for non-offpoint images
;                          incorporate fl_annotate (specialized) 
;       SLF  -  8-oct-1994 Call GAL's <halp.pro> to clean big bear
;       SLF  -  9-oct-1994 Fill out ~256 pixels for better resolution
;                          make white light temporary file.
;       SLF  - 10-oct-1994 took out rotation of white light (FITS arrows flipped??)
;       SLF  - 19-oct-1994 use get1gbo instead of lastgbo 
;	SLF  - 16-jan-1995 protect against no recent gbw case
;	SLF  -  3-mar-1995 expand window (temporary fix)
;       SLF  - 21-jun-1995 He 10830 scaling per J.Gurman/H.Jones
;       SLF  -  5-jul-1995 verify GNT image integrity or go back in time...
;       SLF  - 10-jul-1995 use SOON instead of BBSO (format problem)
;       SLF  - 24-sep-1995 adjusted scaling for soon/gbh/gkm
;			   Call histscale.pro for gbh, gkm
;       SLF  - 12-mar-1996 gkm scaling (may already be scaled)
;       NN   -  9-May-98   shrank He 10830 image slightly so that time be readable.
;-
;---------------------- fl_annotate ----------------------------------------
;+
;	HISTORY:
;	   Procedure developed to annotate the first_light collage 
;	   Written by D. Alexander 5-May-94
;          12-Jul-1994 - (SLF) add character size scaling for z-buffer
;			       parameterize character size
;          15-sep-1994 - (SLF) minor mods - make it part of fl_suntoday
;          19-oct-1994 - (SLF) replace lastgbo call with calls to get1gbo
;          24-sep-1995 - (SLF) scale gkm, rm_darklimb on soon data
;-

pro fl_annotate,gnttime,gkitime,gbhtime,gkmtime,title=title
zbuf_scale=[1.0,.7]
zfact=zbuf_scale(!d.name eq 'Z')
charsize= 1.15 * zfact			; oldsize (1.2)

plots,[0,256],[256,256],/device
plots,[256,256],[256,0],/device
plots,[594,594],[256,0],/device
plots,[594,850],[256,256],/device
plots,[594,594],[594,850],/device
plots,[594,850],[594,594],/device
plots,[0,256],[594,594],/device
plots,[256,256],[594,850],/device
plots,[256,594],[681,681],color=200,/device
plots,[169,169],[256,594],color=200,/device
plots,[681,681],[256,594],color=200,/device
plots,[256,594],[169,169],color=200,/device

xyouts,/norm,0.01,0.71,'!3Nobeyama 17GHz.',charsize=charsize
xyouts,/norm,0.135,0.71,gnttime,charsize=charsize
xyouts,/norm,0.01,0.01,'!3KP He 10830',charsize=charsize
xyouts,/norm,0.135,0.01,gkitime,charsize=charsize
xyouts,/norm,0.71,0.01,title + ' H!7a!3',charsize=charsize
xyouts,/norm,0.835,0.01,gbhtime,charsize=charsize
xyouts,/norm,0.71,0.71,'!3KP Magnet.',charsize=charsize
xyouts,/norm,0.835,0.71,gkmtime,charsize=charsize
xyouts,/norm,0.37,0.93,'!13 The Sun Today!3',charsize=3.0*zfact
return
end

pro fl_suntoday, image, r, g, b, generate=generate, nodist=nodist, debug=debug
debug=keyword_set(debug)
savdev=!d.name
savename='The_SUN_Today'
savetext=concat_dir('$DIR_SITE_DOC',savename + '.doc')
savepath=concat_dir('$DIR_GEN_SHOWPIX','new_data')
savefile=concat_dir(savepath,savename)

if keyword_set(generate) or not file_exist(savefile + '.genx') then begin
   zbuf_scale=[1.0,.75]
   zfact=zbuf_scale(!d.name eq 'Z')
   lastsfd,sindex,sdata,/notv,/half			; recent SXT SFD
   gkiimg=get1gbo('gki', dattim=gkitime,/scale,wind=20)		; KP Intensity
   gkmimg=get1gbo('gkm', head=gkmhead, dattim=gkmtime,/scale,wind=20)		; KP Magnet
   if (wc_where(gkmhead(0),'*FITS STANDARD*'))(0) eq -1 then begin
      message,/info,"Flipping KPNO Magnetograph"
      gkmimg=rotate(gkmimg,7)
   endif
         
   gntt=fmt_tim(!stime)
   gntwindow=20
GNTIMG=BYTSCL(REBIN(DIST(256),512,512)) & GNTTIME=!STIME
;   gntimg=get1gbo('gnt', gntt, dattim=gnttime,wind=gntwindow)	; Nobeyama 17
   ; slf - scan gnts until good image found
   ; gls - took out while loop cause of lack of gnt images
;   while (max(gntimg) eq min(gntimg)) or gntwindow eq 0 do begin
;      message,/info,"Problem with GNT image for " + fmt_tim(gnttime) + "back 1 day..." 
;      gntwindow=gntwindow-1
;      gntt=timegrid(gntt,day=(-1),/string)			 ; back a day
;      gntimg=get1gbo('gnt', gntt, dattim=gnttime,wind=gntwindow)	
;   endwhile

;   gbhimg=get_newsoon(dattim=gbhtime,observ=title)    
;   gbhimg=histscale(gbhimg,histfact=.001,hicut=170)

   gkitime=fmt_tim(gkitime)
   gkmtime=fmt_tim(gkmtime)
   gbhtime=fmt_tim(gbhtime) 
   gnttime=fmt_tim(gnttime)
;   Rescale images to match SXT image

   suncenter = sxt_cen(sindex)
   size_only=(abs((suncenter(1)-600)) ge 25) or $
	     (abs((suncenter(0)-480)) ge 25)
   size_only=1						; *** force size only
;  slf 15-sep-1994 - only resize (not reposition) if offpointed image
;       ***** change diam to alter showpix/firstlight GBO image size ***
;       9-oct-1994 - scale to diam=250 pixels (better resolution)
   diam=250
;
;  clean and scale images (Using D.Alexanders first_light algorithims
;   gntimg=gntcleen(gntimg)
   if debug then stop,'gkiimg...'
;  gkiimg = scale_gbo(sdata,gkiimg,sindex,diam=200,size_only=size_only)
   gki2=bytscl(congrid(gkiimg,235,235),-550,100)
   gkiimg=bytarr(256,256)+min(gki2)
   gkiimg(10,20)=gki2
;   gkiimg = congrid(gkiimg,256,256)
;   gkiimg = bytscl(gkiimg,min=-550,max=100)
   if min(gkmimg) lt 0 then gkmimg = bytscl(gkmimg,min=-100, max=100)
   if debug then stop,'gkmimg...'
   gkmimg=congrid(gkmimg,256,256)
   ;   gkmimg = scale_gbo(sdata,gkmimg,sindex,diam=200, size_only=size_only)
;   gntimg = scale_gbo(sdata,gntimg,sindex,diam=250, size_only=size_only)
   if debug then stop,'gbhimg...'
    gbhimg=indgen(256,256)
;   gbhimg=scale_gbo(sdata,gbhimg,sindex,diam=250,/size_only)
;
;  create collage in Z-buffer (SLF - made Z-buffer to allow background processing)
   set_plot,'z'
   wdef,zz,850,850,/zbuff			; write to Z-buffer 
   loadct,15
   tvscl,sdata,169,169
   sinfo=get_info(sindex)
   topline=strmid(sinfo,5,19)
   botline=strmid(sinfo,26,42)
   xyouts,/norm,0.41,0.26,topline,charsize=1.2*zfact
   xyouts,/norm,0.33,0.23,botline,charsize=1.2*zfact
   edge=(size(gkiimg))(1)/2-128 
   tv,gkiimg(edge:edge+255,edge:edge+255)
 
   edge=(size(gbhimg))(1)/2-128
   tv,gbhimg(edge:edge+255,edge:edge+255),594,0

   edge=(size(gkmimg))(1)/2-128
   tv,gkmimg(edge:edge+255,edge:edge+255),594,594

   edge=(size(gntimg))(1)/2-128
   tvscl,gntimg(edge:edge+255,edge:edge+255),0,594

   if n_elements(title) eq 0 then title=''
   fl_annotate,gnttime,gkitime,gbhtime,gkmtime, title=title
;
;  now read back from the Z-buffer 
   set_plot,'z'
   image=tvrd()
   tvlct,r,g,b,/get			; read back color table

;  now get whitelight			; not in sun_today image for now
   gbwimg=get1gbo('gbw',dattim=dattim)
   wdef,/z,zz,256,256
   if data_chk(gbwimg,/ndimen) ne 0 then begin
      gbwimg=halp(gbwimg,/hist,imgsize=diam)
      gbwimg=histscale(gbwimg,hicut=200)
      rb0p=get_rb0p(dattim)
      gbwimg=rot(rotate(gbwimg,7), rb0p(2)*!radeg, 1,/interp,miss=0)
      gbwimg=rm_darklimb(gbwimg,/setback)
      gbwimg=sigma_filter(gbwimg,n_sig=1)
      outim=bytarr(256,256)
      outim(3,3)=gbwimg
      tv,gbwimg
      xyouts,/dev,10,10,'BBSO Optical ' + fmt_tim(dattim),charsize=.8675
   endif else begin
      outgbw=bytarr(256,256)  
      tv,outgbw
      xyouts,10,128,'NO RECENT DATA',size=2,/dev
      xyouts,/dev,10,10,'BBSO Optical ', charsize=.8675
   endelse
   outgbw=tvrd()
   savegen, file=concat_dir('$DIR_SHOWPIX_SC','gbw_temp.genx'), outgbw

;  Runing at ISAS? then save the data
   if get_logenv('DIR_SITE_SITENAME') eq 'ISAS' then begin
       mk_pix, image, r, g, b, text=savetext, $
         filename=savename, subdir='new_data',/replace, nodist=nodist
   endif
endif else begin
   restgen, file=savefile, image, r, g, b, text=text
endelse

set_plot,savdev				; restore plot device
return
end
